#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <limits.h>

bool jo_write_wav(const char *filename, short numChannels, short bitsPerSample, int sampleRateHz, const void *data, int size) {
	FILE *fp = fopen(filename, "wb");
	if(!fp) {
		return false;
	}
	fwrite("RIFF", 1, 4, fp);
	int length = size + 44 - 8;
	fwrite(&length, 1, 4, fp);
	fwrite("WAVEfmt \x10\x00\x00\x00\x01\x00", 1, 14, fp);
	fwrite(&numChannels, 1, 2, fp);
	fwrite(&sampleRateHz, 1, 4, fp);
	int bpsec = numChannels * sampleRateHz * bitsPerSample / 8;
	fwrite(&bpsec, 1, 4, fp);
	short bpsamp = numChannels * bitsPerSample / 8;
	fwrite(&bpsamp, 1, 2, fp);
	fwrite(&bitsPerSample, 1, 2, fp);
	fwrite("data", 1, 4, fp);
	fwrite(&size, 1, 4, fp);
	fwrite(data, 1, size, fp);
	fclose(fp);
	return true;
}

short *resample(short *in, int numS, int bits) {
	short *out = new short[numS];
	int sh = 16 - bits;
	for(int i = 0; i < numS; ++i) {
		if(sh > 0) {
			int s = in[i];
			int v = s > 0 ? s + (1<<(sh-1)) : s - (1<<(sh-1)); // Rounding
			v = v > SHRT_MAX ? SHRT_MAX : v;
			v = v < SHRT_MIN ? SHRT_MIN : v;
			out[i] = short(v >> sh << sh);
		} else {
			out[i] = in[i];
		}
	}
	return out;
}

int main() {
	FILE *fp = fopen("test.raw", "rb");
	fseek(fp, 0, SEEK_END);
	int size = ftell(fp);
	fseek(fp, 0, SEEK_SET);
	int numSamples = size / 2;
	short *data = new short[numSamples];
	fread(data, 1, size, fp);
	fclose(fp);

	jo_write_wav("test.wav", 1, 16, 44100, data, numSamples*2);
	for(int i = 1; i <= 16; ++i) {
		short *nd = resample(data, numSamples, i);
		char fn[256];
		sprintf(fn, "test%i.wav", i);
		jo_write_wav(fn, 1, 16, 44100, nd, numSamples*2);
		delete [] nd;
	}
	delete [] data;
	return 0;
}


